module Main exposing (..)

import Browser
import Html exposing (Html, button, div, text, input, ul, li, a, h2, p)
import Html.Events exposing (onClick, onInput)
import Html.Attributes exposing (placeholder, value, href)
import Http
import Json.Decode exposing (Decoder, map5, field, int, string, list)


type alias Model =
    { books : Maybe (List Book)
    , searchText : String
    , selectedBook : Maybe Book }

type alias Book =
    { titel : String
    , erscheinungsjahr : Int
    , autor : String
    , genres : String
    , beschreibung : String }

init : () -> (Model, Cmd Msg)
init _ =
    ({ books = Nothing , searchText = "", selectedBook = Nothing }, getBooks)


getBooks : Cmd Msg
getBooks =
    Http.get
        { url = "http://localhost:3000/buecher"
        , expect = Http.expectJson GotBooks booksDecoder
        }


booksDecoder : Decoder (List Book)
booksDecoder =
    list (
        map5 Book
            (field "titel" string)
            (field "erscheinungsjahr" int)
            (field "autor" string)
            (field "genres" string)
            (field "beschreibung" string)
    )

type Msg
    = GotBooks (Result Http.Error (List Book))
    | UpdateSearchText String
    | SelectBook (Maybe Book)


update : Msg -> Model -> (Model, Cmd Msg)
update msg model =
    case msg of
        GotBooks result ->
            case result of
                Ok books ->
                    ({ model | books = Just books }, Cmd.none)

                Err _ ->
                    ({ model | books = Nothing }, Cmd.none)

        UpdateSearchText newText ->
            ({ model | searchText = newText, selectedBook = Nothing }, Cmd.none)

        SelectBook book ->
            ({ model | selectedBook = book }, Cmd.none)


view : Model -> Html Msg
view model =
    div []
        [ input [ placeholder "Search by title", value model.searchText, onInput UpdateSearchText ] []
        , case model.selectedBook of
            Just book ->
                div []
                    [ h2 [] [ text book.titel ]
                    , p [] [ text ("Author: " ++ book.autor) ]
                    , p [] [ text ("Year: " ++ String.fromInt book.erscheinungsjahr) ]
                    , p [] [ text ("Genres: " ++ book.genres) ]
                    , p [] [ text ("Beschreibung: " ++ book.beschreibung) ]
                    , button [ onClick (SelectBook Nothing) ] [ text "Back" ]
                    ]

            Nothing ->
                ul []
                    (case model.books of
                        Just books ->
                            books
                                |> List.filter (\book -> 
                                    String.contains (String.toLower model.searchText) (String.toLower book.titel)
                                )
                                |> List.sortBy .titel
                                |> List.map (\book -> li [] [ a [ href "#", onClick (SelectBook (Just book)) ] [ text book.titel ] ])

                        Nothing ->
                            [ li [] [ text "No books available" ] ]
                    )
        ]


main =
    Browser.element
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view
        }


subscriptions : Model -> Sub Msg
subscriptions _ =
    Sub.none
