module Test1 exposing (..)

import Browser
import Html exposing (Html, button, div, text, input, ul, li)
import Html.Events exposing (onClick, onInput)
import Html.Attributes exposing (placeholder, value)
import Http
import Json.Decode exposing (Decoder, map4, field, int, string, list)
import Url.Builder exposing (absolute)
import Browser.Navigation


-- MODEL

type alias Model =
    { books : Maybe (List Book)
    , searchText : String
    , selectedBooks : List Book }

type alias Book =
    { titel : String
    , erscheinungsjahr : Int
    , autor : String
    , genres : String }


init : () -> (Model, Cmd Msg)
init _ =
    ({ books = Nothing, searchText = "", selectedBooks = [] }, getBooks)


getBooks : Cmd Msg
getBooks =
    Http.get
        { url = "http://localhost:3000/buecher"
        , expect = Http.expectJson GotBooks booksDecoder
        }


booksDecoder : Decoder (List Book)
booksDecoder =
    list
        (map4 Book
            (field "titel" string)
            (field "erscheinungsjahr" int)
            (field "autor" string)
            (field "genres" string)
        )


type Msg
    = GotBooks (Result Http.Error (List Book))
    | UpdateSearchText String
    | SelectBook Book
    | DeselectBook Book
    | ShowSelectedBooks


update : Msg -> Model -> (Model, Cmd Msg)
update msg model =
    case msg of
        GotBooks result ->
            case result of
                Ok l ->
                    ({ model | books = Just l }, Cmd.none)

                Err _ ->
                    ({ model | books = Nothing }, Cmd.none)

        UpdateSearchText newText ->
            ({ model | searchText = newText }, Cmd.none)

        SelectBook book ->
            ({ model | selectedBooks = book :: model.selectedBooks }, Cmd.none)

        DeselectBook book ->
            ({ model | selectedBooks = List.filter ((/=) book) model.selectedBooks }, Cmd.none)

        ShowSelectedBooks ->
            let
                titles =
                    model.selectedBooks
                        |> List.map .titel
                        |> String.join ","
            in
            (model, navigateToSelectedBooksPage titles)


navigateToSelectedBooksPage : String -> Cmd msg
navigateToSelectedBooksPage titles =
    let
        url =
            Url.Builder.absolute
                [ "selected-books" ]
                [ Url.Builder.string "titles" titles ]
    in
    Browser.Navigation.load url




view : Model -> Html Msg
view model =
    div []
        [ input [ placeholder "Search by title", value model.searchText, onInput UpdateSearchText ] []
        , ul []
            (case model.books of
                Just l ->
                    l
                        |> List.filter (\p -> String.contains model.searchText p.titel)
                        |> List.map (\book ->
                            li []
                                [ text book.titel
                                , button [ onClick (SelectBook book) ] [ text "Select" ]
                                , button [ onClick (DeselectBook book) ] [ text "Deselect" ]
                                ]
                        )

                Nothing ->
                    [ li [] [ text "No books available" ] ]
            )
        , div []
            [ button [ onClick ShowSelectedBooks ] [ text "Show Selected Books" ] ]
        ]


main =
    Browser.element
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view
        }


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none
