module Main exposing (main)

import Browser
import Browser.Navigation as Navigation
import Html exposing (Html, div, input, text, button, img)
import Html.Attributes exposing (style, type_, value, class, placeholder)
import Html.Events exposing (onClick, onInput, onBlur)
import Svg exposing (svg, rect, polygon)
import Svg.Attributes exposing (fill, height, points, stroke, strokeWidth, viewBox, width, x, y)
import Url exposing (Url)
import Url.Parser as Parser exposing ((</>), Parser, s, top)
import Platform.Cmd

import Hilfe exposing (view)
import Suche
import Empfehlungen



-- MAIN
main : Program() Model Msg
main =
    Browser.application
        { init = init
        , update = update
        , subscriptions = \_ -> Sub.none
        , view = view
        , onUrlChange = UrlChanged
        , onUrlRequest = LinkClicked
        }


-- MODEL
type alias Model =
    { selectedPart : Part
    , colors : Colors
    , redValue : String
    , greenValue : String
    , blueValue : String
    , key : Navigation.Key
    , currentPage : Page
    , colorInputsVisible : Bool
    , name : String
    , confirmedName : String
    , sucheModel : Suche.Model
    }

type alias Colors =
    { background : String
    , leftWall : String
    , shelf1 : String
    , shelf2 : String
    , shelf3 : String
    , shelf4 : String
    , shelf5 : String
    , rightWall : String
    }

type Msg
    = SelectPart Part
    | ChangeColor
    | UpdateRed String
    | UpdateGreen String
    | UpdateBlue String
    | ChangePage Page
    | UrlChanged Url
    | LinkClicked Browser.UrlRequest
    | ToggleColorInputs
    | Plus
    | UpdateName String
    | ConfirmName
    | SucheM Suche.Msg

type Part
    = Background
    | LeftWall
    | Shelf1
    | Shelf2
    | Shelf3
    | Shelf4
    | Shelf5
    | RightWall

type Page
    = Bookshelf
    | Hilfe
    | Suche
    | Empfehlungen




-- init Funktionen
init : () -> Url -> Navigation.Key -> ( Model, Cmd Msg )
init _ url key =

    let 
        (sucheInitModel, sucheCMD) = Suche.init
    in
    ( { key = key
      , currentPage = parseUrl url
      , colors =
          { background = "rgb(92,60,1)"
          , leftWall = "rgb(122,81,4)"
          , shelf1 = "rgb(138,90,0)"
          , shelf2 = "rgb(138,90,0)"
          , shelf3 = "rgb(138,90,0)"
          , shelf4 = "rgb(138,90,0)"
          , shelf5 = "rgb(138,90,0)"
          , rightWall = "rgb(122,81,4)"
          }
      , redValue = "0"
      , greenValue = "0"
      , blueValue = "0"
      , selectedPart = Background
      , colorInputsVisible = False
      , name = ""
      , confirmedName = ""
      , sucheModel = sucheInitModel
      }
    , Platform.Cmd.map SucheM sucheCMD
    )




-- UPDATE

update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        SelectPart part ->
            ( { model | selectedPart = part }, Cmd.none )

        ChangeColor ->
            let
                color = "rgb(" ++ model.redValue ++ "," ++ model.greenValue ++ "," ++ model.blueValue ++ ")"
            in
            ( { model | colors = updateColor model.selectedPart color model.colors }, Cmd.none )

        UpdateRed redValue ->
            ( { model | redValue = redValue }, Cmd.none )

        UpdateGreen greenValue ->
            ( { model | greenValue = greenValue }, Cmd.none )

        UpdateBlue blueValue ->
            ( { model | blueValue = blueValue }, Cmd.none )

        ChangePage page ->
            ( { model | currentPage = page }, Navigation.pushUrl model.key (toUrl page) )

        UrlChanged url ->
            ( { model | currentPage = parseUrl url }, Cmd.none )

        LinkClicked urlRequest ->
            case urlRequest of
                Browser.Internal url ->
                    ( model, Navigation.pushUrl model.key (Url.toString url) )

                Browser.External href ->
                    ( model, Navigation.load href )
        ToggleColorInputs ->
            ( { model | colorInputsVisible = not model.colorInputsVisible }, Cmd.none )
        Plus ->
            ( { model | colorInputsVisible = not model.colorInputsVisible }, Cmd.none )

        UpdateName newName ->
           ({model | name = newName} ,Cmd.none)

        ConfirmName ->
            ({model | confirmedName = model.name}, Cmd.none)


        SucheM msg2 ->
            let 
                (m, c) = Suche.update msg2 model.sucheModel
            in
                ({model | sucheModel = m}, Platform.Cmd.map SucheM c)

updateColor : Part -> String -> Colors -> Colors
updateColor part color colors =
    case part of
        Background ->
            { colors | background = color }
        LeftWall ->
            { colors | leftWall = color }
        Shelf1 ->
            { colors | shelf1 = color }
        Shelf2 ->
            { colors | shelf2 = color }
        Shelf3 ->
            { colors | shelf3 = color }
        Shelf4 ->
            { colors | shelf4 = color }
        Shelf5 ->
            { colors | shelf5 = color }
        RightWall ->
            { colors | rightWall = color }

darkenColor : String -> String -> String -> String
darkenColor red green blue =
    let
        darken value =
            let
                intValue = String.toInt value |> Maybe.withDefault 0
                darkenedValue = max 0 (intValue - 30)
            in
            String.fromInt darkenedValue
    in
    "rgb(" ++ darken red ++ "," ++ darken green ++ "," ++ darken blue ++ ")"


-- ROUTING
toUrl : Page -> String
toUrl page =
    case page of
        Bookshelf ->
            "/"

        Hilfe ->
            "/hilfe"

        Suche ->
            "/suche"

        Empfehlungen ->
            "/empfehlungen"

parseUrl : Url -> Page
parseUrl url =
    case Parser.parse pageParser url of
        Just page ->
            page

        Nothing ->
            Bookshelf

pageParser : Parser (Page -> Page) Page
pageParser =
    Parser.oneOf
        [ Parser.map Bookshelf top
        , Parser.map Hilfe (s "hilfe")
        , Parser.map Suche (s "suche")
        , Parser.map Empfehlungen (s "empfehlungen")
        ]

-- COLORING
extractRed : String -> String
extractRed color =
    String.split "," (String.dropLeft 4 (String.dropRight 1 color))
        |> List.head
        |> Maybe.withDefault "0"

extractGreen : String -> String
extractGreen color =
    String.split "," (String.dropLeft 4 (String.dropRight 1 color))
        |> List.drop 1
        |> List.head
        |> Maybe.withDefault "0"

extractBlue : String -> String
extractBlue color =
    String.split "," (String.dropLeft 4 (String.dropRight 1 color))
        |> List.drop 2
        |> List.head
        |> Maybe.withDefault "0"

-- VIEW

view : Model -> Browser.Document Msg
view model =
    { title = "Bookshelf_online"
    , body =
        [ div [style "overflow" "hidden"
            , style "height" "100%"
            , style "width" "100%"]
            [ div [ class "navbar has-background-black", style "display" "flex", style "justify-content" "center", style "align-items" "center", style "height" "60px" ]
                [ div [ class "navbar-item", style "color" "gray", style "cursor" "pointer", onClick (ChangePage Bookshelf) ]
                    [ text "Bookshelf" ]
                , div [ class "navbar-item", style "color" "gray", style "cursor" "pointer", onClick (ChangePage Suche) ]
                    [ text "Suche" ]
                , div [ class "navbar-item", style "color" "gray", style "cursor" "pointer", onClick (ChangePage Empfehlungen) ]
                    [ text "Empfehlungen" ]
                , div [ class "navbar-item", style "color" "gray", style "cursor" "pointer", onClick (ChangePage Hilfe) ]
                    [ text "Hilfe" ]
                ]
            ]
        , div []
            [ case model.currentPage of
                Bookshelf ->
                    bookshelfView model
                Hilfe ->
                    Hilfe.view
                Suche ->
                    Html.map SucheM (Suche.view model.sucheModel)
                Empfehlungen ->
                    Empfehlungen.view
            ]
        ]
    }



bookshelfView : Model -> Html Msg
bookshelfView model =
    div 
        [ style "height" "100%"
        , style "width" "100%"
        , style "display" "flex"
        , style "justify-content" "center"
        , style "align-items" "center"
        , style "background-image" "url('Hintergrund3.jpg')"
        , style "background-size" "cover"
        , style "background-repeat" "no-repeat"
        , style "background-position" "center"
        , style "background-attachment" "fixed"
        , style "overflow" "hidden"
        , style "padding-top" "150px"
        , style "overflow" "hidden"
        , style "position" "fixed"
        ]
        [ div [ style "position" "relative", style "display" "flex", style "justify-content" "center", style "align-items" "center", style "height" "calc(100vh - 60px)" ] -- Hauptcontainer für Regal und Farbeingabe
            [ div [ style "display" "flex", style "justify-content" "center", style "align-items" "center" ] -- Container für SVG und Farbeingabe nebeneinander
                [   div [style "position" "absolute", style "top" "200px", style "left" "-350px", style "width" "350px", style "height" "750px"]
                        [ img [ Html.Attributes.src "pixel_pflanze.png", Html.Attributes.alt "Pflanze" ] []
                        ]
                    , div [style "position" "absolute", style "top" "-50px", style "left" "1250px", style "width" "170px", style "height" "170px"]
                        [ img [ Html.Attributes.src "mc_bild-1.png", Html.Attributes.alt "Gemälde" ] []
                        ]
                    ,svg [ viewBox "0 0 240 160", width "1200px", height "900px" ] -- SVG mit größerer fester Größe
                    [ rect
                        [ x "20", y "0", width "180", height "130", fill model.colors.background, stroke (if model.selectedPart == Background then "black" else "none"), strokeWidth "2", onClick (SelectPart Background) ]
                        []
                    , polygon
                        [ points "0,10 20,0 20,120 0,130", fill model.colors.leftWall, stroke (if model.selectedPart == LeftWall then "black" else "none"), strokeWidth "2", onClick (SelectPart LeftWall) ]
                        []
                    , polygon
                        [ points "20,0 220,0 200,10 0,10", fill model.colors.shelf1, stroke (if model.selectedPart == Shelf1 then "black" else "none"), strokeWidth "2", onClick (SelectPart Shelf1) ]
                        []
                    , rect
                        [ x "0", y "10", width "200", height "5", fill (darkenColor (extractRed model.colors.shelf1) (extractGreen model.colors.shelf1) (extractBlue model.colors.shelf1)) ]
                        []
                    , polygon
                        [ points "20,30 220,30 200,40 0,40", fill model.colors.shelf2, stroke (if model.selectedPart == Shelf2 then "black" else "none"), strokeWidth "2", onClick (SelectPart Shelf2) ]
                        []
                    , rect
                        [ x "0", y "40", width "200", height "5", fill (darkenColor (extractRed model.colors.shelf2) (extractGreen model.colors.shelf2) (extractBlue model.colors.shelf2)) ]
                        []
                    , polygon
                        [ points "20,60 220,60 200,70 0,70", fill model.colors.shelf3, stroke (if model.selectedPart == Shelf3 then "black" else "none"), strokeWidth "2", onClick (SelectPart Shelf3) ]
                        []
                    , rect
                        [ x "0", y "70", width "200", height "5", fill (darkenColor (extractRed model.colors.shelf3) (extractGreen model.colors.shelf3) (extractBlue model.colors.shelf3)) ]
                        []
                    , polygon
                        [ points "20,90 220,90 200,100 0,100", fill model.colors.shelf4, stroke (if model.selectedPart == Shelf4 then "black" else "none"), strokeWidth "2", onClick (SelectPart Shelf4) ]
                        []
                    , rect
                        [ x "0", y "100", width "200", height "5", fill (darkenColor (extractRed model.colors.shelf4) (extractGreen model.colors.shelf4) (extractBlue model.colors.shelf4)) ]
                        []
                    , polygon
                        [ points "20,120 220,120 200,130 0,130", fill model.colors.shelf5, stroke (if model.selectedPart == Shelf5 then "black" else "none"), strokeWidth "2", onClick (SelectPart Shelf5) ]
                        []
                    , rect
                        [ x "0", y "130", width "200", height "5", fill (darkenColor (extractRed model.colors.shelf5) (extractGreen model.colors.shelf5) (extractBlue model.colors.shelf5)) ]
                        []
                    , polygon
                        [ points "220,0 220,125 200,135 200,10", fill model.colors.rightWall, stroke (if model.selectedPart == RightWall then "black" else "none"), strokeWidth "2", onClick (SelectPart RightWall) ]
                        []
                    ]
            , div [ style "margin-left" "20px", style "position" "absolute", style "right" "20px", style "top" "20px" ] -- Container für Farbeingabe und Button
                [ button 
                    [ onClick ToggleColorInputs
                    , style "border-radius" "9px"
                    , style "cursor" "pointer"
                    , style "padding" "0" -- Entferne die Padding, damit das Bild den gesamten Platz im Button ausfüllen kann
                    , style "width" "64px"
                    , style "height" "64px"
                    , style "background-image" "url('Farbe.png')"
                    , style "background-size" "contain"
                    , style "background-repeat" "no-repeat"
                    , style "background-position" "center"
                    , style "position" "absolute"
                    , style "top" "120px"
                    , style "right" "-100px"
                    ]
                    []
                , if model.colorInputsVisible then
                      div [ style "padding" "10px", style "position" "absolute", style "top" "80px", style "right" "-350px", style "font-size" "25px" ]
                          [ div [ style "margin-bottom" "10px", style "color" "red", style "font-family" "Georgia", style "font-size" "20px"]
                              [ input [ type_ "number", value model.redValue, onInput UpdateRed ] []
                              , text " Rot"
                              ]
                          , div [ style "margin-bottom" "10px", style "color" "green", style "font-family" "Georgia", style "font-size" "20px" ]
                              [ input [ type_ "number", value model.greenValue, onInput UpdateGreen ] []
                              , text " Grün"
                              ]
                          , div [ style "margin-bottom" "10px", style "color" "blue" , style "font-family" "Georgia", style "font-size" "20px"]
                              [ input [ type_ "number", value model.blueValue, onInput UpdateBlue ] []
                              , text " Blau"
                              ]
                          , div []
                              [ button [ onClick ChangeColor
                                        , style "background-image" "linear-gradient(to right, red, green, blue)"
                                        , style "font-family" "Georgia" 
                                        , style "border-radius" "10px"
                                        , style "cursor" "pointer"
                                        , style "font-size" "20px"
                                        , style "color" "black"] [ text "Farbe ändern" ] ]
                      ]
                  else
                      text ""
                ]
                , div [ style "margin-top" "20px" ] -- Neuer Bereich für den zusätzlichen Button
                [ button 
                    [ onClick ToggleColorInputs
                    , style "border-radius" "9px"
                    , style "cursor" "pointer"
                    , style "padding" "0" -- Entferne die Padding, damit das Bild den gesamten Platz im Button ausfüllen kann
                    , style "width" "64px"
                    , style "height" "64px"
                    , style "background-image" "url('Maus.png')"
                    , style "background-size" "contain"
                    , style "background-repeat" "no-repeat"
                    , style "background-position" "center"
                    , style "position" "absolute"
                    , style "top" "275px"
                    , style "right" "-80px"
                    ]
                    []
                ]
                , div [ style "margin-top" "20px" ] -- Neuer Bereich für den zusätzlichen Button
                [ button 
                    [ onClick ToggleColorInputs
                    , style "border-radius" "9px"
                    , style "cursor" "pointer"
                    , style "padding" "0" -- Entferne die Padding, damit das Bild den gesamten Platz im Button ausfüllen kann
                    , style "width" "64px"
                    , style "height" "64px"
                    , style "background-image" "url('Plus.png')"
                    , style "background-size" "contain"
                    , style "background-repeat" "no-repeat"
                    , style "background-position" "center"
                    , style "position" "absolute"
                    , style "top" "400px"
                    , style "right" "-80px"
                    ]
                    []
                ]
                ,div []
                [ input
                    [ placeholder "Mein Bücherregal"
                    , value model.name
                    , onInput UpdateName
                    , onBlur ConfirmName
                    , style "border" "none"
                    , style "outline" "none"
                    , style "font-size" "75px"
                    , style "font-family" "Georgia"
                    , style "text-align" "center"
                    , style "background-color" "transparent"
                    , style "position" "absolute"
                    , style "top" "-95px"
                    , style "right" "200px"
                    ]
                    []
                ]    
            ]
        ]
    ]