module TestHome exposing (main)

import Browser
import Browser.Navigation as Navigation
import Html exposing (button, div, text)
import Html.Attributes exposing (class)
import Html.Events exposing (onClick)
import Url exposing (Url)
import Url.Parser as Parser exposing ((</>), Parser, s, top)

import Hilfe
import Suche
import Empfehlungen


-- MODEL

type alias Model =
    { key : Navigation.Key
    , currentPage : Page
    }

type Page
    = Hilfe
    | Suche
    | Empfehlungen


-- INIT

init : () -> Url -> Navigation.Key -> ( Model, Cmd Msg )
init _ url key =
    ( { key = key, currentPage = parseUrl url }
    , Cmd.none
    )


-- UPDATE

type Msg
    = ChangePage Page
    | UrlChanged Url
    | LinkClicked Browser.UrlRequest

update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        ChangePage page ->
            ( { model | currentPage = page }
            , Navigation.pushUrl model.key (toUrl page)
            )

        UrlChanged url ->
            ( { model | currentPage = parseUrl url }
            , Cmd.none
            )

        LinkClicked urlRequest ->
            case urlRequest of
                Browser.Internal url ->
                    ( model, Navigation.pushUrl model.key (Url.toString url) )

                Browser.External href ->
                    ( model, Navigation.load href )


-- VIEW

view : Model -> Browser.Document Msg
view model =
    let
        pageContent =
            case model.currentPage of
                Hilfe ->
                    Hilfe.view

                Suche ->
                    Suche.view

                Empfehlungen ->
                    Empfehlungen.view
    in
    { title = "Test Application"
    , body =
        [ div []
            [ div [ class "menu" ]
                [ button [ onClick (ChangePage Hilfe) ] [ text "Hilfe" ]
                , button [ onClick (ChangePage Suche) ] [ text "Suche" ]
                , button [ onClick (ChangePage Empfehlungen) ] [ text "Empfehlungen" ]
                ]
            , div [ class "content" ]
                [ pageContent ]
            ]
        ]
    }


-- ROUTING

toUrl : Page -> String
toUrl page =
    case page of
        Hilfe ->
            "/"

        Suche ->
            "/suche"

        Empfehlungen ->
            "/empfehlungen"

parseUrl : Url -> Page
parseUrl url =
    case Parser.parse pageParser url of
        Just page ->
            page

        Nothing ->
            Hilfe

pageParser : Parser (Page -> Page) Page
pageParser =
    Parser.oneOf
        [ Parser.map Hilfe top
        , Parser.map Suche (s "suche")
        , Parser.map Empfehlungen (s "empfehlungen")
        ]


-- MAIN
main : Program() Model Msg
main =
    Browser.application
        { init = init
        , update = update
        , subscriptions = \_ -> Sub.none
        , view = view
        , onUrlChange = UrlChanged
        , onUrlRequest = LinkClicked
        }
